@@ -69,6 +69,7 @@ var (
 	web                      bool
 	noExit                   bool
 	_, useCloudEngine        = os.LookupEnv("DAGGER_CLOUD_ENGINE")
+	enableScaleOut           bool
 
 	dotOutputFilePath string
 	dotFocusField     string
@@ -140,6 +141,7 @@ func init() {
 		watchCmd,
 		configCmd,
 		checksCmd,
+		generateCmd,
 		moduleInitCmd,
 		moduleInstallCmd,
 		moduleUnInstallCmd,
@@ -346,6 +348,10 @@ func installGlobalFlags(flags *pflag.FlagSet) {
 	flags.BoolVar(&useCloudEngine, "cloud", useCloudEngine, "Run in a Dagger Cloud Engine")
 	flags.Lookup("cloud").Hidden = true
 
+	// this flag enables scale-out for a few commands, e.g. checks, generate
+	flags.BoolVar(&enableScaleOut, "scale-out", false, "Enable scale-out to cloud engines for each check or generate executed")
+	flags.Lookup("scale-out").Hidden = true
+
 	for _, fl := range []string{
 		"workdir",
 		"dot-output",
