@@ -446,26 +446,32 @@ type Activity struct {
 
 func (activity *Activity) Intervals(now time.Time) iter.Seq[Interval] {
 	return func(yield func(Interval) bool) {
-		var lastIval *Interval
+		var latestEnd time.Time
+		yieldRunning := func() {
+			runningStart := activity.EarliestRunning
+			if latestEnd.After(runningStart) {
+				runningStart = latestEnd
+			}
+			if runningStart.Before(now) {
+				yield(Interval{Start: runningStart, End: now})
+			}
+		}
 		for _, ival := range activity.CompletedIntervals {
-			lastIval = &ival
 			if !activity.EarliestRunning.IsZero() &&
 				activity.EarliestRunning.Before(ival.Start) {
-				if !yield(Interval{Start: activity.EarliestRunning, End: now}) {
-					return
-				}
-				break
+				yieldRunning()
+				return
 			}
 			if !yield(ival) {
 				return
 			}
-		}
-		if !activity.EarliestRunning.IsZero() &&
-			(lastIval == nil || activity.EarliestRunning.After(lastIval.End)) {
-			if !yield(Interval{Start: activity.EarliestRunning, End: now}) {
-				return
+			if ival.End.After(latestEnd) {
+				latestEnd = ival.End
 			}
 		}
+		if !activity.EarliestRunning.IsZero() {
+			yieldRunning()
+		}
 	}
 }
 
