@@ -25,6 +25,11 @@ func ContextWithDebugMode(ctx context.Context, debug bool) context.Context {
 	return toggleBaggage(ctx, debugBaggageKey, debug)
 }
 
+func IsDebug(ctx context.Context) bool {
+	bag := baggage.FromContext(ctx)
+	return bag.Member(debugBaggageKey).Value() == "true"
+}
+
 // ContextWithColorMode enables or disables color mode in the given context's
 // OpenTelemetry baggage.
 func ContextWithColorMode(ctx context.Context, noColor bool) context.Context {
