@@ -23,3 +23,6 @@ var GoSum []byte
 
 //go:embed engineconn/*.go querybuilder/marshal.go querybuilder/querybuilder.go go.mod go.sum client.go dagger.gen.go telemetry/*.go
 var GoSDK embed.FS
+
+//go:embed dagger.gen.go
+var GoDagGen []byte
